@@ -43,11 +43,6 @@ src_sqlite <- function(dbname = ":memory:", ...) {
   featUuid <- pkgNeeded("RSQLite", "2.3.7.9014", FALSE)
   if (featUuid) {
     RSQLite::initExtension(db = con, extension = "uuid")
-    message(
-      "RSQLite version has enabled accelerating ",
-      "docdb_create() and docdb_update() functions ",
-      "when used with value = <NDJSON file name>."
-    )
   }
 
   # set timeout for concurrency to 10s
